import os
import re
import requests
import subprocess
import sys
from packaging.version import parse

PYSISU_ROOT = os.path.abspath(
    os.path.join(__file__, os.pardir, os.pardir, os.pardir)
)

def main() -> int:
    most_recent_common_ancestor = (
        subprocess.run(
            ["git", "merge-base", "HEAD", "master"],
            capture_output=True,
            check=True,
        )
        .stdout.decode("utf-8")
        .rstrip("\n")
    )

    pysisu_diff = subprocess.run(
        ["git", "diff", "--quiet", most_recent_common_ancestor, "./pysisu"]
    ).returncode

    if not pysisu_diff:
        return 0

    current_master_commit = (
        subprocess.run(
            ["git", "rev-parse", "master"], capture_output=True, check=True
        )
        .stdout.decode("utf-8")
        .rstrip("\n")
    )

    master_pysisu_version = subprocess.run(
        ["git", "show", f"{current_master_commit}:./pysisu/version.py"],
        capture_output=True,
        check=True,
    ).stdout.decode("utf-8")

    local_pysisu_version = subprocess.run(
        ["cat", os.path.join(PYSISU_ROOT, "pysisu", "version.py")],
        capture_output=True,
        check=True,
    ).stdout.decode("utf-8")

    pypi_latest_version = requests.get(
        "https://pypi.org/pypi/pysisu/json", headers={"Connection": "close"}
    ).json()["info"]["version"]

    # find version between quotes from: __version__ = "0.13.2"
    version_regex = '"([^"]*)"'

    master_version_matches = re.findall(version_regex, master_pysisu_version)
    local_version_matches = re.findall(version_regex, local_pysisu_version)

    if len(local_version_matches) != 1:
        raise SyntaxError(
            "Local core/pysisu/pysisu/version.py file contains invalid semantic version."
        )

    if len(master_version_matches) != 1:
        raise SyntaxError(
            "Master core/pysisu/pysisu/version.py file contains invalid semantic version."
        )

    local_version_match = local_version_matches[0]
    master_version_match = master_version_matches[0]

    if parse(local_version_match) <= parse(pypi_latest_version):
        print("Pysisu version must be greater than latest version on PyPI.")
        return 1
    if parse(local_version_match) < parse(master_version_match):
        print(
            "Pysisu version should not be lower than current master version. "
            "Please merge or rebase master and update version if necessary."
        )
        return 1
    return 0


if __name__ == "__main__":
    sys.exit(main())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: options/annotations.proto, options/openapiv2.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    Dict,
    List,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class Scheme(betterproto.Enum):
    """
    Scheme describes the schemes supported by the OpenAPI Swagger and Operation
    objects.
    """

    UNKNOWN = 0
    HTTP = 1
    HTTPS = 2
    WS = 3
    WSS = 4


class JsonSchemaJsonSchemaSimpleTypes(betterproto.Enum):
    UNKNOWN = 0
    ARRAY = 1
    BOOLEAN = 2
    INTEGER = 3
    NULL = 4
    NUMBER = 5
    OBJECT = 6
    STRING = 7


class SecuritySchemeType(betterproto.Enum):
    """
    The type of the security scheme. Valid values are "basic", "apiKey" or
    "oauth2".
    """

    TYPE_INVALID = 0
    TYPE_BASIC = 1
    TYPE_API_KEY = 2
    TYPE_OAUTH2 = 3


class SecuritySchemeIn(betterproto.Enum):
    """The location of the API key. Valid values are "query" or "header"."""

    IN_INVALID = 0
    IN_QUERY = 1
    IN_HEADER = 2


class SecuritySchemeFlow(betterproto.Enum):
    """
    The flow used by the OAuth2 security scheme. Valid values are "implicit",
    "password", "application" or "accessCode".
    """

    FLOW_INVALID = 0
    FLOW_IMPLICIT = 1
    FLOW_PASSWORD = 2
    FLOW_APPLICATION = 3
    FLOW_ACCESS_CODE = 4


@dataclass(eq=False, repr=False)
class Swagger(betterproto.Message):
    """
    `Swagger` is a representation of OpenAPI v2 specification's Swagger object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#swaggerObject Example:  option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_swagger) = {    info:
    {      title: "Echo API";      version: "1.0";      description: "";
    contact: {        name: "gRPC-Gateway project";        url:
    "https://github.com/grpc-ecosystem/grpc-gateway";        email:
    "none@example.com";      };      license: {        name: "BSD 3-Clause
    License";        url: "https://github.com/grpc-ecosystem/grpc-
    gateway/blob/master/LICENSE.txt";      };    };    schemes: HTTPS;
    consumes: "application/json";    produces: "application/json";  };
    """

    swagger: str = betterproto.string_field(1)
    """
    Specifies the OpenAPI Specification version being used. It can be used by
    the OpenAPI UI and other clients to interpret the API listing. The value
    MUST be "2.0".
    """

    info: "Info" = betterproto.message_field(2)
    """
    Provides metadata about the API. The metadata can be used by the clients if
    needed.
    """

    host: str = betterproto.string_field(3)
    """
    The host (name or ip) serving the API. This MUST be the host only and does
    not include the scheme nor sub-paths. It MAY include a port. If the host is
    not included, the host serving the documentation is to be used (including
    the port). The host does not support path templating.
    """

    base_path: str = betterproto.string_field(4)
    """
    The base path on which the API is served, which is relative to the host. If
    it is not included, the API is served directly under the host. The value
    MUST start with a leading slash (/). The basePath does not support path
    templating. Note that using `base_path` does not change the endpoint paths
    that are generated in the resulting OpenAPI file. If you wish to use
    `base_path` with relatively generated OpenAPI paths, the `base_path` prefix
    must be manually removed from your `google.api.http` paths and your code
    changed to serve the API from the `base_path`.
    """

    schemes: List["Scheme"] = betterproto.enum_field(5)
    """
    The transfer protocol of the API. Values MUST be from the list: "http",
    "https", "ws", "wss". If the schemes is not included, the default scheme to
    be used is the one used to access the OpenAPI definition itself.
    """

    consumes: List[str] = betterproto.string_field(6)
    """
    A list of MIME types the APIs can consume. This is global to all APIs but
    can be overridden on specific API calls. Value MUST be as described under
    Mime Types.
    """

    produces: List[str] = betterproto.string_field(7)
    """
    A list of MIME types the APIs can produce. This is global to all APIs but
    can be overridden on specific API calls. Value MUST be as described under
    Mime Types.
    """

    responses: Dict[str, "Response"] = betterproto.map_field(
        10, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    An object to hold responses that can be used across operations. This
    property does not define global responses for all operations.
    """

    security_definitions: "SecurityDefinitions" = betterproto.message_field(11)
    """
    Security scheme definitions that can be used across the specification.
    """

    security: List["SecurityRequirement"] = betterproto.message_field(12)
    """
    A declaration of which security schemes are applied for the API as a whole.
    The list of values describes alternative security schemes that can be used
    (that is, there is a logical OR between the security requirements).
    Individual operations can override this definition.
    """

    external_docs: "ExternalDocumentation" = betterproto.message_field(14)
    """Additional external documentation."""

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(15, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class Operation(betterproto.Message):
    """
    `Operation` is a representation of OpenAPI v2 specification's Operation
    object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#operationObject Example:  service
    EchoService {    rpc Echo(SimpleMessage) returns (SimpleMessage) {
    option (google.api.http) = {        get: "/v1/example/echo/{id}"      };
    option (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_operation) = {
    summary: "Get a message.";        operation_id: "getMessage";        tags:
    "echo";        responses: {          key: "200"            value: {
    description: "OK";          }        }      };    }  }
    """

    tags: List[str] = betterproto.string_field(1)
    """
    A list of tags for API documentation control. Tags can be used for logical
    grouping of operations by resources or any other qualifier.
    """

    summary: str = betterproto.string_field(2)
    """
    A short summary of what the operation does. For maximum readability in the
    swagger-ui, this field SHOULD be less than 120 characters.
    """

    description: str = betterproto.string_field(3)
    """
    A verbose explanation of the operation behavior. GFM syntax can be used for
    rich text representation.
    """

    external_docs: "ExternalDocumentation" = betterproto.message_field(4)
    """Additional external documentation for this operation."""

    operation_id: str = betterproto.string_field(5)
    """
    Unique string used to identify the operation. The id MUST be unique among
    all operations described in the API. Tools and libraries MAY use the
    operationId to uniquely identify an operation, therefore, it is recommended
    to follow common programming naming conventions.
    """

    consumes: List[str] = betterproto.string_field(6)
    """
    A list of MIME types the operation can consume. This overrides the consumes
    definition at the OpenAPI Object. An empty value MAY be used to clear the
    global definition. Value MUST be as described under Mime Types.
    """

    produces: List[str] = betterproto.string_field(7)
    """
    A list of MIME types the operation can produce. This overrides the produces
    definition at the OpenAPI Object. An empty value MAY be used to clear the
    global definition. Value MUST be as described under Mime Types.
    """

    responses: Dict[str, "Response"] = betterproto.map_field(
        9, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    The list of possible responses as they are returned from executing this
    operation.
    """

    schemes: List["Scheme"] = betterproto.enum_field(10)
    """
    The transfer protocol for the operation. Values MUST be from the list:
    "http", "https", "ws", "wss". The value overrides the OpenAPI Object
    schemes definition.
    """

    deprecated: bool = betterproto.bool_field(11)
    """
    Declares this operation to be deprecated. Usage of the declared operation
    should be refrained. Default value is false.
    """

    security: List["SecurityRequirement"] = betterproto.message_field(12)
    """
    A declaration of which security schemes are applied for this operation. The
    list of values describes alternative security schemes that can be used
    (that is, there is a logical OR between the security requirements). This
    definition overrides any declared top-level security. To remove a top-level
    security declaration, an empty array can be used.
    """

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(13, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class Header(betterproto.Message):
    """
    `Header` is a representation of OpenAPI v2 specification's Header object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#headerObject
    """

    description: str = betterproto.string_field(1)
    """`Description` is a short description of the header."""

    type: str = betterproto.string_field(2)
    """
    The type of the object. The value MUST be one of "string", "number",
    "integer", or "boolean". The "array" type is not supported.
    """

    format: str = betterproto.string_field(3)
    """`Format` The extending format for the previously mentioned type."""

    default: str = betterproto.string_field(6)
    """
    `Default` Declares the value of the header that the server will use if none
    is provided. See: https://tools.ietf.org/html/draft-fge-json-schema-
    validation-00#section-6.2. Unlike JSON Schema this value MUST conform to
    the defined type for the header.
    """

    pattern: str = betterproto.string_field(13)
    """
    'Pattern' See https://tools.ietf.org/html/draft-fge-json-schema-
    validation-00#section-5.2.3.
    """


@dataclass(eq=False, repr=False)
class Response(betterproto.Message):
    """
    `Response` is a representation of OpenAPI v2 specification's Response
    object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#responseObject
    """

    description: str = betterproto.string_field(1)
    """
    `Description` is a short description of the response. GFM syntax can be
    used for rich text representation.
    """

    schema: "Schema" = betterproto.message_field(2)
    """
    `Schema` optionally defines the structure of the response. If `Schema` is
    not provided, it means there is no content to the response.
    """

    headers: Dict[str, "Header"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    `Headers` A list of headers that are sent with the response. `Header` name
    is expected to be a string in the canonical format of the MIME header key
    See: https://golang.org/pkg/net/textproto/#CanonicalMIMEHeaderKey
    """

    examples: Dict[str, str] = betterproto.map_field(
        4, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    `Examples` gives per-mimetype response examples. See:
    https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#example-object
    """

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(5, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class Info(betterproto.Message):
    """
    `Info` is a representation of OpenAPI v2 specification's Info object. See:
    https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#infoObject Example:  option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_swagger) = {    info:
    {      title: "Echo API";      version: "1.0";      description: "";
    contact: {        name: "gRPC-Gateway project";        url:
    "https://github.com/grpc-ecosystem/grpc-gateway";        email:
    "none@example.com";      };      license: {        name: "BSD 3-Clause
    License";        url: "https://github.com/grpc-ecosystem/grpc-
    gateway/blob/master/LICENSE.txt";      };    };    ...  };
    """

    title: str = betterproto.string_field(1)
    """The title of the application."""

    description: str = betterproto.string_field(2)
    """
    A short description of the application. GFM syntax can be used for rich
    text representation.
    """

    terms_of_service: str = betterproto.string_field(3)
    """The Terms of Service for the API."""

    contact: "Contact" = betterproto.message_field(4)
    """The contact information for the exposed API."""

    license: "License" = betterproto.message_field(5)
    """The license information for the exposed API."""

    version: str = betterproto.string_field(6)
    """
    Provides the version of the application API (not to be confused with the
    specification version).
    """

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(7, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class Contact(betterproto.Message):
    """
    `Contact` is a representation of OpenAPI v2 specification's Contact object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#contactObject Example:  option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_swagger) = {    info:
    {      ...      contact: {        name: "gRPC-Gateway project";        url:
    "https://github.com/grpc-ecosystem/grpc-gateway";        email:
    "none@example.com";      };      ...    };    ...  };
    """

    name: str = betterproto.string_field(1)
    """The identifying name of the contact person/organization."""

    url: str = betterproto.string_field(2)
    """
    The URL pointing to the contact information. MUST be in the format of a
    URL.
    """

    email: str = betterproto.string_field(3)
    """
    The email address of the contact person/organization. MUST be in the format
    of an email address.
    """


@dataclass(eq=False, repr=False)
class License(betterproto.Message):
    """
    `License` is a representation of OpenAPI v2 specification's License object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#licenseObject Example:  option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_swagger) = {    info:
    {      ...      license: {        name: "BSD 3-Clause License";        url:
    "https://github.com/grpc-ecosystem/grpc-gateway/blob/master/LICENSE.txt";
    };      ...    };    ...  };
    """

    name: str = betterproto.string_field(1)
    """The license name used for the API."""

    url: str = betterproto.string_field(2)
    """
    A URL to the license used for the API. MUST be in the format of a URL.
    """


@dataclass(eq=False, repr=False)
class ExternalDocumentation(betterproto.Message):
    """
    `ExternalDocumentation` is a representation of OpenAPI v2 specification's
    ExternalDocumentation object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#externalDocumentationObject
    Example:  option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_swagger) = {    ...
    external_docs: {      description: "More about gRPC-Gateway";      url:
    "https://github.com/grpc-ecosystem/grpc-gateway";    }    ...  };
    """

    description: str = betterproto.string_field(1)
    """
    A short description of the target documentation. GFM syntax can be used for
    rich text representation.
    """

    url: str = betterproto.string_field(2)
    """
    The URL for the target documentation. Value MUST be in the format of a URL.
    """


@dataclass(eq=False, repr=False)
class Schema(betterproto.Message):
    """
    `Schema` is a representation of OpenAPI v2 specification's Schema object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#schemaObject
    """

    json_schema: "JsonSchema" = betterproto.message_field(1)
    discriminator: str = betterproto.string_field(2)
    """
    Adds support for polymorphism. The discriminator is the schema property
    name that is used to differentiate between other schema that inherit this
    schema. The property name used MUST be defined at this schema and it MUST
    be in the required property list. When used, the value MUST be the name of
    this schema or any schema that inherits it.
    """

    read_only: bool = betterproto.bool_field(3)
    """
    Relevant only for Schema "properties" definitions. Declares the property as
    "read only". This means that it MAY be sent as part of a response but MUST
    NOT be sent as part of the request. Properties marked as readOnly being
    true SHOULD NOT be in the required list of the defined schema. Default
    value is false.
    """

    external_docs: "ExternalDocumentation" = betterproto.message_field(5)
    """Additional external documentation for this schema."""

    example: str = betterproto.string_field(6)
    """
    A free-form property to include an example of an instance for this schema
    in JSON. This is copied verbatim to the output.
    """


@dataclass(eq=False, repr=False)
class JsonSchema(betterproto.Message):
    """
    `JSONSchema` represents properties from JSON Schema taken, and as used, in
    the OpenAPI v2 spec. This includes changes made by OpenAPI v2. See:
    https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#schemaObject See also:
    https://cswr.github.io/JsonSchema/spec/basic_types/,
    https://github.com/json-schema-org/json-schema-spec/blob/master/schema.json
    Example:  message SimpleMessage {    option
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_schema) = {
    json_schema: {        title: "SimpleMessage"        description: "A simple
    message."        required: ["id"]      }    };    // Id represents the
    message identifier.    string id = 1; [
    (grpc.gateway.protoc_gen_openapiv2.options.openapiv2_field) = {
    description: "The unique identifier of the simple message."        }];  }
    """

    ref: str = betterproto.string_field(3)
    """
    Ref is used to define an external reference to include in the message. This
    could be a fully qualified proto message reference, and that type must be
    imported into the protofile. If no message is identified, the Ref will be
    used verbatim in the output. For example:  `ref:
    ".google.protobuf.Timestamp"`.
    """

    title: str = betterproto.string_field(5)
    """The title of the schema."""

    description: str = betterproto.string_field(6)
    """A short description of the schema."""

    default: str = betterproto.string_field(7)
    read_only: bool = betterproto.bool_field(8)
    example: str = betterproto.string_field(9)
    """
    A free-form property to include a JSON example of this field. This is
    copied verbatim to the output swagger.json. Quotes must be escaped. This
    property is the same for 2.0 and 3.0.0 https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/3.0.0.md#schemaObject
    https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#schemaObject
    """

    multiple_of: float = betterproto.double_field(10)
    maximum: float = betterproto.double_field(11)
    """
    Maximum represents an inclusive upper limit for a numeric instance. The
    value of MUST be a number,
    """

    exclusive_maximum: bool = betterproto.bool_field(12)
    minimum: float = betterproto.double_field(13)
    """
    minimum represents an inclusive lower limit for a numeric instance. The
    value of MUST be a number,
    """

    exclusive_minimum: bool = betterproto.bool_field(14)
    max_length: int = betterproto.uint64_field(15)
    min_length: int = betterproto.uint64_field(16)
    pattern: str = betterproto.string_field(17)
    max_items: int = betterproto.uint64_field(20)
    min_items: int = betterproto.uint64_field(21)
    unique_items: bool = betterproto.bool_field(22)
    max_properties: int = betterproto.uint64_field(24)
    min_properties: int = betterproto.uint64_field(25)
    required: List[str] = betterproto.string_field(26)
    array: List[str] = betterproto.string_field(34)
    """Items in 'array' must be unique."""

    type: List["JsonSchemaJsonSchemaSimpleTypes"] = betterproto.enum_field(35)
    format: str = betterproto.string_field(36)
    """`Format`"""

    enum: List[str] = betterproto.string_field(46)
    """
    Items in `enum` must be unique https://tools.ietf.org/html/draft-fge-json-
    schema-validation-00#section-5.5.1
    """

    field_configuration: "JsonSchemaFieldConfiguration" = betterproto.message_field(
        1001
    )
    """
    Additional field level properties used when generating the OpenAPI v2 file.
    """

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(48, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class JsonSchemaFieldConfiguration(betterproto.Message):
    """
    'FieldConfiguration' provides additional field level properties used when
    generating the OpenAPI v2 file. These properties are not defined by
    OpenAPIv2, but they are used to control the generation.
    """

    path_param_name: str = betterproto.string_field(47)
    """
    Alternative parameter name when used as path parameter. If set, this will
    be used as the complete parameter name when this field is used as a path
    parameter. Use this to avoid having auto generated path parameter names for
    overlapping paths.
    """


@dataclass(eq=False, repr=False)
class Tag(betterproto.Message):
    """
    `Tag` is a representation of OpenAPI v2 specification's Tag object. See:
    https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#tagObject
    """

    description: str = betterproto.string_field(2)
    """
    A short description for the tag. GFM syntax can be used for rich text
    representation.
    """

    external_docs: "ExternalDocumentation" = betterproto.message_field(3)
    """Additional external documentation for this tag."""


@dataclass(eq=False, repr=False)
class SecurityDefinitions(betterproto.Message):
    """
    `SecurityDefinitions` is a representation of OpenAPI v2 specification's
    Security Definitions object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#securityDefinitionsObject A
    declaration of the security schemes available to be used in the
    specification. This does not enforce the security schemes on the operations
    and only serves to provide the relevant details for each scheme.
    """

    security: Dict[str, "SecurityScheme"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    A single security scheme definition, mapping a "name" to the scheme it
    defines.
    """


@dataclass(eq=False, repr=False)
class SecurityScheme(betterproto.Message):
    """
    `SecurityScheme` is a representation of OpenAPI v2 specification's Security
    Scheme object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#securitySchemeObject Allows the
    definition of a security scheme that can be used by the operations.
    Supported schemes are basic authentication, an API key (either as a header
    or as a query parameter) and OAuth2's common flows (implicit, password,
    application and access code).
    """

    type: "SecuritySchemeType" = betterproto.enum_field(1)
    """
    The type of the security scheme. Valid values are "basic", "apiKey" or
    "oauth2".
    """

    description: str = betterproto.string_field(2)
    """A short description for security scheme."""

    name: str = betterproto.string_field(3)
    """
    The name of the header or query parameter to be used. Valid for apiKey.
    """

    in_: "SecuritySchemeIn" = betterproto.enum_field(4)
    """
    The location of the API key. Valid values are "query" or "header". Valid
    for apiKey.
    """

    flow: "SecuritySchemeFlow" = betterproto.enum_field(5)
    """
    The flow used by the OAuth2 security scheme. Valid values are "implicit",
    "password", "application" or "accessCode". Valid for oauth2.
    """

    authorization_url: str = betterproto.string_field(6)
    """
    The authorization URL to be used for this flow. This SHOULD be in the form
    of a URL. Valid for oauth2/implicit and oauth2/accessCode.
    """

    token_url: str = betterproto.string_field(7)
    """
    The token URL to be used for this flow. This SHOULD be in the form of a
    URL. Valid for oauth2/password, oauth2/application and oauth2/accessCode.
    """

    scopes: "Scopes" = betterproto.message_field(8)
    """
    The available scopes for the OAuth2 security scheme. Valid for oauth2.
    """

    extensions: Dict[
        str, "betterproto_lib_google_protobuf.Value"
    ] = betterproto.map_field(9, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)


@dataclass(eq=False, repr=False)
class SecurityRequirement(betterproto.Message):
    """
    `SecurityRequirement` is a representation of OpenAPI v2 specification's
    Security Requirement object. See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#securityRequirementObject Lists
    the required security schemes to execute this operation. The object can
    have multiple security schemes declared in it which are all required (that
    is, there is a logical AND between the schemes). The name used for each
    property MUST correspond to a security scheme declared in the Security
    Definitions.
    """

    security_requirement: Dict[
        str, "SecurityRequirementSecurityRequirementValue"
    ] = betterproto.map_field(1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE)
    """
    Each name must correspond to a security scheme which is declared in the
    Security Definitions. If the security scheme is of type "oauth2", then the
    value is a list of scope names required for the execution. For other
    security scheme types, the array MUST be empty.
    """


@dataclass(eq=False, repr=False)
class SecurityRequirementSecurityRequirementValue(betterproto.Message):
    """
    If the security scheme is of type "oauth2", then the value is a list of
    scope names required for the execution. For other security scheme types,
    the array MUST be empty.
    """

    scope: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class Scopes(betterproto.Message):
    """
    `Scopes` is a representation of OpenAPI v2 specification's Scopes object.
    See: https://github.com/OAI/OpenAPI-
    Specification/blob/3.0.0/versions/2.0.md#scopesObject Lists the available
    scopes for an OAuth2 security scheme.
    """

    scope: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    Maps between a name of a scope to a short description of it (as the value
    of the property).
    """



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import re
import urllib
from typing import Tuple


def pathjoin(*args) -> str:
    return '/'.join(s.strip('/') for s in args)


def build_url(base_url, path, args_dict):
    url_parts = list(urllib.parse.urlparse(base_url))
    url_parts[2] = path
    url_parts[4] = urllib.parse.urlencode(args_dict)
    return urllib.parse.urlunparse(url_parts)


def semver_parse(version: str) -> Tuple[int, int, int]:
    pattern = r"([1-9]\d*|0)\.([1-9]\d*|0)\.([1-9]\d*|0)"
    match = re.compile(pattern).match(version)
    if match is None:
        raise SyntaxError(
            f"{version} is not a valid semantic version"
        )
    return tuple(int(x) for x in match.groups())



import json
import os


CURR_PATH = os.path.dirname(os.path.abspath(__file__))
INPUT_SNAPSHOT_DIR = os.path.join(CURR_PATH, "input_snapshots")
OUTPUT_SNAPSHOT_DIR = os.path.join(CURR_PATH, "output_snapshots")
SNAPSHOT_DICT = {
    "time compare": "auto_time_compare_kda_result.json",
    "group compare": "auto_group_compare_kda_result.json",
    "general performance": "auto_gen_perf_kda_result.json",
    "trend": "auto_trend_analysis_result.json",
    "time compare has more": "auto_time_compare_kda_result_has_more.json",
    "time compare has more doubled": "auto_time_compare_kda_result_has_more_doubled.json",
    "list analyses": "auto_active_analysis_list.json",
    "list metrics": "auto_active_metrics_list.json",
    "list data sources": "auto_data_source_list.json",
    "list datasets": "auto_dataset_list.json",
    "get analysis filters": "mixed_filter_expression.json",
    "list projects": "auto_project_list.json",
    "list project analyses": "auto_project_analysis_list.json",
}


def load_snapshot(snapshot_name: str) -> dict:
    snapshot = os.path.join(INPUT_SNAPSHOT_DIR, SNAPSHOT_DICT[snapshot_name])
    with open(snapshot, "r") as file:
        return json.loads(file.read())


def pyapp_api_snapshots_to_pysisu() -> None:
    """
    Recursively walks the pyapp API snapshots directory
    to get snapshots, copies them all into pysisu/tests/input_snapshots, and
    removes any pysisu snapshots that don't correspond to a pyapp snapshot.

    If not run from Sisu core repo, do nothing.
    """

    if os.environ.get("SKIP_SNAPSHOT_MIGRATION") == "true":
        return

    try:
        from pysisu.pyapp_snapshot_directory import PYAPP_SNAPSHOT_DIR
    except ModuleNotFoundError:
        return 0

    # keep track of snapshots without a pyapp counterpart
    snapshots_to_keep = {
        "auto_time_compare_kda_result_has_more.json",
        "auto_time_compare_kda_result_has_more_doubled.json",
    }

    # check for existence of pyapp snapshots in the same directory
    if not os.path.isdir(PYAPP_SNAPSHOT_DIR):
        raise FileNotFoundError(
            1,
            f"pyapp api snapshots directory no longer in {PYAPP_SNAPSHOT_DIR}, please update pyapp_snapshot_directory.py with new location",
        )

    # write pyapp snapshots to input_snapshots recursively & flatten dir structure
    for subdir, _, files in os.walk(PYAPP_SNAPSHOT_DIR):
        for file in files:
            snapshots_to_keep.add(file)
            with open(os.path.join(subdir, file), "r") as pyapp_snapshot:
                pyapp_contents = pyapp_snapshot.read()
                if os.path.exists(os.path.join(INPUT_SNAPSHOT_DIR, file)):
                    with open(
                        os.path.join(INPUT_SNAPSHOT_DIR, file), "r"
                    ) as pysisu_snapshot:
                        if pysisu_snapshot.read() == pyapp_contents:
                            continue
                    with open(
                        os.path.join(INPUT_SNAPSHOT_DIR, file), "w"
                    ) as pysisu_snapshot:
                        print(
                            f"Current snapshot {file} needs to be updated. Updating..."
                        )
                        pysisu_snapshot.write(pyapp_contents)
                        continue

                # else file doesn't exist in INPUT_SNAPSHOT_DIR
                with open(
                    os.path.join(INPUT_SNAPSHOT_DIR, file), "w"
                ) as pysisu_snapshot:
                    print(
                        f"Snapshot {file} does not yet exist in pysisu. Writing..."
                    )
                    pysisu_snapshot.write(pyapp_contents)

    for subdir, _, files in os.walk(INPUT_SNAPSHOT_DIR):
        for file in files:
            if file not in snapshots_to_keep:
                print(
                    f"Snapshot {file} no longer exists in pyapp. Deleting..."
                )
                os.remove(os.path.join(INPUT_SNAPSHOT_DIR, file))



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from dataclasses import dataclass
from enum import Enum, auto
from typing import List, Type


@dataclass
class Table:
    header: List["HeaderColumn"]
    rows: List["Row"]

    def to_csv(self, delimiter : str = ',') -> str:
        header = delimiter.join(x.column_name for x in self.header)
        rows = '\n'.join(row.to_tabular_str(delimiter) for row in self.rows)
        return f'{header}\n{rows}'


@dataclass
class Row:
    def to_tabular_str(self, delimiter: str = ',') -> str:
        variables = []
        for x in vars(self).values():
            if x is None:
                variables.append('')
            elif isinstance(x, (int, float)):
                variables.append(x)
            else:
                variables.append(f"'{x}'")
        return delimiter.join([str(x).replace('\n', ' ') for x in variables])

    def __str__(self):
        return self.to_tabular_str()


@dataclass
class HeaderColumn:
    column_name: str
    column_type: Type


class LatestAnalysisResultsFormats(Enum):
    TABLE = auto()
    PROTO = auto()



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
from pysisu import PySisu

API_KEY = os.environ.get('SISU_API_KEY')
ANALYSIS_ID = 15245

sisu = PySisu(API_KEY)
filters = sisu.get_filters(ANALYSIS_ID)
print(filters)
resp = sisu.set_filters(ANALYSIS_ID, filters.to_dict())
print(resp)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: v1/api.public.proto
# plugin: python-betterproto
import builtins
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class AnalysisType(betterproto.Enum):
    """Type of Analysis eg. KEY_DRIVER."""

    ANALYSIS_TYPE_UNKNOWN = 0
    ANALYSIS_TYPE_GENERAL_PERFORMANCE = 1
    ANALYSIS_TYPE_TREND = 2
    ANALYSIS_TYPE_TIME_COMPARE = 3
    ANALYSIS_TYPE_GROUP_COMPARE = 4


class SqlDataType(betterproto.Enum):
    """Represents a datatype of a specific dimension."""

    SQL_DATA_TYPE_UNKNOWN = 0
    SQL_DATA_TYPE_STRING = 1
    SQL_DATA_TYPE_FLOAT = 2
    SQL_DATA_TYPE_TIMESTAMP = 3
    SQL_DATA_TYPE_INT = 4
    SQL_DATA_TYPE_BOOLEAN = 5


class ExpressionBasicExpressionOperator(betterproto.Enum):
    OPERATOR_UNKNOWN = 0
    OPERATOR_LT = 1
    OPERATOR_EQ = 2
    OPERATOR_LTE = 3
    OPERATOR_GTE = 4
    OPERATOR_GT = 5
    OPERATOR_NE = 6
    OPERATOR_LIKE = 7
    OPERATOR_NOT_LIKE = 8
    OPERATOR_IS = 9
    OPERATOR_IS_NOT = 10


class AnalysisResultRunStatus(betterproto.Enum):
    """Status of running an analysis."""

    RUN_STATUS_UNKNOWN = 0
    RUN_STATUS_IN_FLIGHT = 1
    """Analysis is currently running."""

    RUN_STATUS_FAILED = 2
    """Analysis finished running but had errors."""

    RUN_STATUS_COMPLETED = 3
    """Analysis ran successfully."""


class AnalysisResultRunType(betterproto.Enum):
    """Either SCHEDULED or MANUAL."""

    RUN_TYPE_UNKNOWN = 0
    RUN_TYPE_SCHEDULED = 1
    RUN_TYPE_MANUAL = 2


class KeyDriverAnalysisResultSegmentConfidenceLevel(betterproto.Enum):
    """
    Reflects how interesting/valuable Sisu determines a specific segment to be.
    """

    CONFIDENCE_LEVEL_UNKNOWN = 0
    CONFIDENCE_LEVEL_HIGH = 1
    CONFIDENCE_LEVEL_MEDIUM = 2
    CONFIDENCE_LEVEL_LOW = 3


class KeyDriverAnalysisResultSegmentCustomCalculationType(betterproto.Enum):
    """Custom calculation types."""

    CUSTOM_CALCULATION_TYPE_UNKNOWN = 0
    CUSTOM_CALCULATION_TYPE_SUM = 1
    CUSTOM_CALCULATION_TYPE_AVERAGE = 2
    CUSTOM_CALCULATION_TYPE_COUNT = 3
    CUSTOM_CALCULATION_TYPE_CHANGE_IN_SUM = 4
    CUSTOM_CALCULATION_TYPE_CHANGE_IN_AVERAGE = 5
    CUSTOM_CALCULATION_TYPE_CHANGE_IN_COUNT = 6
    CUSTOM_CALCULATION_TYPE_CHANGE_IN_SIZE = 7
    CUSTOM_CALCULATION_TYPE_RATE_EFFECT = 8
    CUSTOM_CALCULATION_TYPE_MIX_SHIFT_EFFECT = 9
    CUSTOM_CALCULATION_TYPE_MIX_SHIFT_EFFECT_B_NORMALIZED = 10
    CUSTOM_CALCULATION_TYPE_NET_EFFECT = 11
    CUSTOM_CALCULATION_TYPE_NET_EFFECT_B_NORMALIZED = 12
    CUSTOM_CALCULATION_TYPE_RELATIVE_CHANGE = 13
    CUSTOM_CALCULATION_TYPE_RELATIVE_PERCENT_CHANGE_OF_METRIC = 14
    CUSTOM_CALCULATION_TYPE_CHANGE_IN_WEIGHTED_CATEGORICAL_GP_IMPACT = 15


class UnitsConfigUnitType(betterproto.Enum):
    UNIT_TYPE_UNKNOWN = 0
    UNIT_TYPE_CUSTOM = 1
    UNIT_TYPE_PERCENT = 2
    UNIT_TYPE_CURRENCY = 3
    UNIT_TYPE_BPS = 4
    UNIT_TYPE_NUMBER = 5


class MetricDesiredDirection(betterproto.Enum):
    """Type of metric direction."""

    DESIRED_DIRECTION_UNKNOWN = 0
    DESIRED_DIRECTION_INCREASE = 1
    DESIRED_DIRECTION_DECREASE = 2


class MetricMetricType(betterproto.Enum):
    """
    Type of metric calculation that is used to evaluate the metric's dimension.
    """

    METRIC_TYPE_UNKNOWN = 0
    METRIC_TYPE_AVERAGE = 1
    """Average of a single metric dimension(e.g., average order value)."""

    METRIC_TYPE_SUM = 2
    """Sum of a single metric dimension(e.g., total revenue)."""

    METRIC_TYPE_WEIGHTED_SUM = 3
    """
    Sum of a metric dimension weighted by a weight dimension(e.g., price per
    share).
    """

    METRIC_TYPE_WEIGHTED_AVERAGE = 4
    """
    Average of a metric dimension weighted by a weight dimension(e.g., price
    per share).
    """

    METRIC_TYPE_CATEGORICAL_COUNT = 5
    """
    Count of a matching condition in a metric dimension(e.g., number of
    churns).
    """

    METRIC_TYPE_CATEGORICAL_RATE = 6
    """
    Rate of a matching condition in a metric dimension(e.g., churn rate).
    """

    METRIC_TYPE_COUNT_DISTINCT = 7
    """
    Count of a matching condition in a metric dimension(e.g., number of
    churns).
    """

    METRIC_TYPE_NUMERICAL_COUNT = 8
    """
    Count of the rows of a single metric dimension(e.g., number of orders).
    """

    METRIC_TYPE_NUMERICAL_RATE = 9
    """
    A metric dimension divided by a denominator dimension(e.g., lead conversion
    rate).
    """


class DataSourceDataSourceType(betterproto.Enum):
    DATA_SOURCE_TYPE_UNKNOWN = 0
    DATA_SOURCE_TYPE_REDSHIFT = 1
    DATA_SOURCE_TYPE_POSTGRESQL = 2
    DATA_SOURCE_TYPE_SNOWFLAKE = 3
    DATA_SOURCE_TYPE_SQLSERVER = 4
    DATA_SOURCE_TYPE_BIGQUERY = 5
    DATA_SOURCE_TYPE_VERTICA = 6
    DATA_SOURCE_TYPE_AWSATHENA = 7
    DATA_SOURCE_TYPE_SPARK = 8
    DATA_SOURCE_TYPE_DATABRICKS = 9
    DATA_SOURCE_TYPE_SAP = 10
    DATA_SOURCE_TYPE_CSV = 11
    DATA_SOURCE_TYPE_DBTCLOUD = 12


class DatasetDatasetType(betterproto.Enum):
    DATASET_TYPE_UNKNOWN = 0
    DATASET_TYPE_TABLE = 1
    """Data is sourced from a single table."""

    DATASET_TYPE_QUERY = 2
    """Data is sourced from a user-defined query."""


@dataclass(eq=False, repr=False)
class GetAnalysisFiltersRequest(betterproto.Message):
    """Request parameters for analysis filters endpoints"""

    id: Optional[int] = betterproto.message_field(1, wraps=betterproto.TYPE_INT64)
    """Analysis id."""


@dataclass(eq=False, repr=False)
class SetAnalysisFiltersRequest(betterproto.Message):
    """Request parameters for analysis filters endpoints"""

    id: Optional[int] = betterproto.message_field(1, wraps=betterproto.TYPE_INT64)
    """Analysis id."""

    filter_expression: "Expression" = betterproto.message_field(2)
    """
    A filter expression must be one of 'and'/'or' [expression] with min of 2
    expressions. or a single basicCondition.
    """


@dataclass(eq=False, repr=False)
class SetAnalysisFiltersResponse(betterproto.Message):
    """Response with empty message."""

    pass


@dataclass(eq=False, repr=False)
class GetAnalysisFiltersResponse(betterproto.Message):
    """Response for analysis filter endpoint."""

    filter_expression: "Expression" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Expression(betterproto.Message):
    """An Expression which would facilitate building a filter expression."""

    and_: "ExpressionAndExpression" = betterproto.message_field(
        1, group="expression_value"
    )
    or_: "ExpressionOrExpression" = betterproto.message_field(
        2, group="expression_value"
    )
    basic_condition: "ExpressionBasicExpression" = betterproto.message_field(
        3, group="expression_value"
    )


@dataclass(eq=False, repr=False)
class ExpressionAndExpression(betterproto.Message):
    """A conjunction of boolean expressions  e.g. `<expr1> AND <expr2>`."""

    expressions: List["Expression"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ExpressionOrExpression(betterproto.Message):
    """
    A conjunction of boolean expressions  e.g. `<expr1> OR <expr2>`. An OR
    expression must no have AND expression.
    """

    expressions: List["Expression"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ExpressionLiteral(betterproto.Message):
    """literal represent the data type and value of a dimension."""

    bool: builtins.bool = betterproto.bool_field(1, group="value")
    int: builtins.int = betterproto.int64_field(2, group="value")
    string: str = betterproto.string_field(3, group="value")
    float: builtins.float = betterproto.double_field(4, group="value")
    timestamp: datetime = betterproto.message_field(5, group="value")


@dataclass(eq=False, repr=False)
class ExpressionBasicExpression(betterproto.Message):
    """
    An basic boolean expression e.g. `"sale_price" > 100`. {   "dimensionName"
    : "sale_price",   "operator" : "OPERATOR_LT",   "literal" : {     "int" :
    100   } }
    """

    operator: "ExpressionBasicExpressionOperator" = betterproto.enum_field(1)
    dimension_name: str = betterproto.string_field(2)
    """A SQL identifier name, e.g 'my_column'"""

    null_literal: "betterproto_lib_google_protobuf.Empty" = betterproto.message_field(
        3, group="literal_value"
    )
    literal: "ExpressionLiteral" = betterproto.message_field(4, group="literal_value")


@dataclass(eq=False, repr=False)
class AnalysisRunResultsRequest(betterproto.Message):
    """Request parameters for get analysis results."""

    limit: Optional[int] = betterproto.message_field(1, wraps=betterproto.TYPE_UINT64)
    """
    A limit on the number of objects to be returned, between 1 and 10000.
    Default value is 10000.
    """

    starting_after: Optional[int] = betterproto.message_field(
        2, wraps=betterproto.TYPE_INT64
    )
    """
    starting_after is an object ID that defines your place in the list. For
    instance, if you make a analysis list request and receive 100, ending with
    id = 89, your subsequent call can include starting_after=89 in order to
    fetch the next page of the list.
    """

    confidence_gte: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    filter by confidence levels of greater than equal to HIGH, MEDIUM or LOW
    """

    id: Optional[int] = betterproto.message_field(4, wraps=betterproto.TYPE_INT64)
    """Analysis Id."""


@dataclass(eq=False, repr=False)
class AnalysesListRequest(betterproto.Message):
    """Request parameters for get analysis list."""

    analysis_type: "AnalysisType" = betterproto.enum_field(1)
    """
    What type of analyses to include in the results. If not set all types will
    be returned.
    """

    limit: Optional[int] = betterproto.message_field(2, wraps=betterproto.TYPE_UINT64)
    """
    A limit on the number of objects to be returned, between 1 and 10000.
    Default value is 10000.
    """

    starting_after: Optional[int] = betterproto.message_field(
        3, wraps=betterproto.TYPE_INT64
    )
    """
    starting_after is an object ID that defines your place in the list. For
    instance, if you make a analysis list request and receive 100, ending with
    id = 89, your subsequent call can include starting_after=89 in order to
    fetch the next page of the list.
    """


@dataclass(eq=False, repr=False)
class AnalysesListResponse(betterproto.Message):
    """AnalysesListResponse provides list of Analyses."""

    analyses: List["Analysis"] = betterproto.message_field(1)
    """List of analyses."""

    pagination_hints: "PaginationHints" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Analysis(betterproto.Message):
    """Provides detailed information about an analysis."""

    id: int = betterproto.int64_field(1)
    """Analysis id."""

    name: str = betterproto.string_field(2)
    """Analysis name."""

    type: "AnalysisType" = betterproto.enum_field(3)
    """Type of Analysis eg. TYPE_KEY_DRIVER."""

    created_at: datetime = betterproto.message_field(4)
    """Timestamp when the analysis was created."""

    metric_id: int = betterproto.uint64_field(5)
    """Metric id corresponding to the analysis."""

    project_id: int = betterproto.uint64_field(6)
    """Project id corresponding to the analysis."""

    application_url: str = betterproto.string_field(7)
    """
    Link to the live sisu analysis this represents. ex:
    vip.sisudata.com/projects/{id}/analysis/{id}
    """


@dataclass(eq=False, repr=False)
class RunAnalysisRequest(betterproto.Message):
    """Request payload for execute analysis workflow."""

    id: int = betterproto.int64_field(1)
    """Analysis id."""


@dataclass(eq=False, repr=False)
class RunAnalysisResponse(betterproto.Message):
    """Response message execute analysis workflow returns empty message"""

    pass


@dataclass(eq=False, repr=False)
class PaginationHints(betterproto.Message):
    """
    Pagination hints which indicate if more data is available
    next_starting_cursor indicate the next id to be used for starting_after
    pagination parameter
    """

    has_more: bool = betterproto.bool_field(1)
    next_starting_cursor: Optional[int] = betterproto.message_field(
        2, wraps=betterproto.TYPE_INT64
    )


@dataclass(eq=False, repr=False)
class AnalysisRunResultsResponse(betterproto.Message):
    """Response payload for get LatestAnalysisResult."""

    analysis_result: "AnalysisResult" = betterproto.message_field(1)
    """Analysis Result."""

    pagination_hints: "PaginationHints" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class KdaSummaryCard(betterproto.Message):
    """Information specific to a single card in a summary."""

    card_label: str = betterproto.string_field(1)
    """Summary card label (bolded in UI)."""

    category_filter: Optional[str] = betterproto.message_field(
        2, wraps=betterproto.TYPE_STRING
    )
    """
    String representation of the filter that produces the metric category.
    """

    summary_value: Optional[float] = betterproto.double_field(
        4, optional=True, group="_summary_value"
    )
    """Value of the metric for the specified subgroup."""

    match_size: Optional[float] = betterproto.double_field(
        5, optional=True, group="_match_size"
    )
    """
    Number of rows matching the filter expression in the specified subgroup.
    """

    total_size: Optional[float] = betterproto.double_field(
        6, optional=True, group="_total_size"
    )
    """Number of total rows in the specified subgroup."""

    min: Optional[float] = betterproto.double_field(7, optional=True, group="_min")
    """Minimum of metric dimension column in the specified subgroup."""

    max: Optional[float] = betterproto.double_field(8, optional=True, group="_max")
    """Maximum of metric dimension column in the specified subgroup."""

    median: Optional[float] = betterproto.double_field(
        9, optional=True, group="_median"
    )
    """Median of metric dimension column in the specified subgroup."""

    average: Optional[float] = betterproto.double_field(
        10, optional=True, group="_average"
    )
    """Average of metric dimension column in the specified subgroup."""

    weighted_average: Optional[float] = betterproto.double_field(
        11, optional=True, group="_weighted_average"
    )
    """
    Weighted average of metric dimension column against specified weight column
    in the specified subgroup.
    """

    sum: Optional[float] = betterproto.double_field(12, optional=True, group="_sum")
    """Sum of metric dimension column in the specified subgroup."""

    weighted_sum: Optional[float] = betterproto.double_field(
        13, optional=True, group="_weighted_sum"
    )
    """
    Weighted sum of metric dimension column against specified weight column in
    the specified subgroup.
    """

    weight: Optional[float] = betterproto.double_field(
        14, optional=True, group="_weight"
    )
    """Sum of weight dimension column in the specified subgroup."""

    total_numerator: Optional[float] = betterproto.double_field(
        15, optional=True, group="_total_numerator"
    )
    """Sum of numerator dimension column in the specified subgroup."""

    total_denominator: Optional[float] = betterproto.double_field(
        16, optional=True, group="_total_denominator"
    )
    """Sum of denominator dimension column in the specified subgroup."""


@dataclass(eq=False, repr=False)
class GroupComparisonSummaryCard(betterproto.Message):
    """Summary card details specific to a group comparison."""

    group_a_card: "KdaSummaryCard" = betterproto.message_field(1)
    """Summary card for group A."""

    group_b_card: "KdaSummaryCard" = betterproto.message_field(2)
    """Summary card for group B."""

    group_a_filter: str = betterproto.string_field(3)
    """Filter that produces group A."""

    group_b_filter: str = betterproto.string_field(4)
    """Filter that produces group B."""

    percent_change: float = betterproto.double_field(5)
    """Percent change of metric value from group A to group B."""


@dataclass(eq=False, repr=False)
class TimeComparisonSummaryCard(betterproto.Message):
    """Summary card details specific to a time comparison."""

    current_period_card: "KdaSummaryCard" = betterproto.message_field(1)
    """Summary card for current period."""

    previous_period_card: "KdaSummaryCard" = betterproto.message_field(2)
    """Summary card for previous period."""

    current_period: "TimeRange" = betterproto.message_field(3)
    """Current period start (inclusive) and end (exclusive) datetimes."""

    previous_period: "TimeRange" = betterproto.message_field(4)
    """Previous period start (inclusive) and end (exclusive) datetimes."""

    percent_change: float = betterproto.double_field(5)
    """
    Percent change of metric value from previous period to current period.
    """


@dataclass(eq=False, repr=False)
class TrendSummaryCard(betterproto.Message):
    """Summary card details of an individual overall trend."""

    card_label: str = betterproto.string_field(1)
    """Summary card label (bolded in UI)."""

    slope: float = betterproto.double_field(2)
    """Slope of the metric over the trend period."""

    percent_change: float = betterproto.double_field(11)
    """
    Percent increase or decrease of metric dimension column in the first time
    unit of a trend.
    """

    denominator_label: str = betterproto.string_field(12)
    """Analysis summary period; visible for trend detection analyses."""


@dataclass(eq=False, repr=False)
class TrendSummaryCardInfo(betterproto.Message):
    """Summary card details specific to a trend detection."""

    current_period: "TrendSummaryCard" = betterproto.message_field(1)
    """Most-recent overall trend summary card."""

    previous_period: "TrendSummaryCard" = betterproto.message_field(2)
    """Second-most-recent overall trend summary card."""


@dataclass(eq=False, repr=False)
class KdaSummaryCardInfo(betterproto.Message):
    """Summary card details of a KDA result."""

    metric_type_label: str = betterproto.string_field(1)
    """Metric label present for KDAs; example: "Numerical Count"""

    general_performance_card: "KdaSummaryCard" = betterproto.message_field(
        2, group="card_type"
    )
    """General performance KDA summary card."""

    group_comparison_card: "GroupComparisonSummaryCard" = betterproto.message_field(
        3, group="card_type"
    )
    """Group comparison KDA summary card."""

    time_comparison_card: "TimeComparisonSummaryCard" = betterproto.message_field(
        4, group="card_type"
    )
    """Time comparison KDA summary card."""


@dataclass(eq=False, repr=False)
class AnalysisResult(betterproto.Message):
    """Provides details of an analysis run."""

    id: int = betterproto.int64_field(1)
    """
    The run ID Analysis run. Run ID's are unique across analyses. example:
    102940
    """

    run_status: "AnalysisResultRunStatus" = betterproto.enum_field(2)
    """Indicates if analysis run completed successfully or not."""

    requested_at: datetime = betterproto.message_field(3)
    """Time at which run was kicked off."""

    completed_at: datetime = betterproto.message_field(4)
    """Time at which analysis run completed."""

    run_type: "AnalysisResultRunType" = betterproto.enum_field(5)
    key_driver_analysis_result: "KeyDriverAnalysisResult" = betterproto.message_field(
        6, group="run_result"
    )
    trend_analysis_result: "TrendAnalysisResult" = betterproto.message_field(
        7, group="run_result"
    )
    metric_id: int = betterproto.uint64_field(8)
    """Metric ID for the analysis."""

    application_url: str = betterproto.string_field(9)
    """
    Link to the live sisu analysis this represents. ex:
    vip.sisudata.com/projects/{id}/analysis/{id}
    """

    result_version: Optional[str] = betterproto.string_field(
        10, optional=True, group="_result_version"
    )
    """
    result_version id is used to identify new incremental results while
    analysis is still in flight.
    """


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResult(betterproto.Message):
    """Provides details of a key driver analysis run."""

    summary_card: "KdaSummaryCardInfo" = betterproto.message_field(1)
    """Summary information for the key driver analysis."""

    time_comparison: "KeyDriverAnalysisResultTimeComparison" = (
        betterproto.message_field(6, group="comparison")
    )
    """
    If subtype is TIME_COMPARISON, metadata about the time periods that are
    compared.
    """

    group_comparison: "KeyDriverAnalysisResultGroupComparison" = (
        betterproto.message_field(7, group="comparison")
    )
    """
    If subtype is GROUP_COMPARISON metadata about the groups that are being
    compared.
    """

    general_performance: "KeyDriverAnalysisResultGeneralPerformance" = (
        betterproto.message_field(8, group="comparison")
    )
    """If the subtype is General_Performance."""

    segments: List["KeyDriverAnalysisResultSegment"] = betterproto.message_field(10)
    """Array of the segments selected by the key driver algorithm."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultTimeComparison(betterproto.Message):
    """
    If subtype is TIME_COMPARISON, metadata about the time periods that are
    compared.
    """

    previous_period: "TimeRange" = betterproto.message_field(1)
    """The earlier of the two periods being compared."""

    recent_period: "TimeRange" = betterproto.message_field(2)
    """The more recent of the two periods being compared."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultGroupComparison(betterproto.Message):
    """
    If subtype is GROUP_COMPARISON metadata about the groups that are being
    compared.
    """

    group_a: "KeyDriverAnalysisResultGroupComparisonGroupDescription" = (
        betterproto.message_field(1)
    )
    """The first group."""

    group_b: "KeyDriverAnalysisResultGroupComparisonGroupDescription" = (
        betterproto.message_field(2)
    )
    """The second group."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultGroupComparisonGroupDescription(betterproto.Message):
    name: str = betterproto.string_field(1)
    """The user-defined name corresponding to the first group."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultGeneralPerformance(betterproto.Message):
    """If subtype is General Performance."""

    pass


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultSegment(betterproto.Message):
    """Segment of a key driver analysis run."""

    id: int = betterproto.int64_field(1)
    """Unique ID corresponding to each segment, unique per analysis run."""

    factors: Dict[str, "Factor"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    The factors that define this segment, represented as a map of Dimension to
    Value.
    """

    group_comparison: "KeyDriverAnalysisResultSegmentGroupComparisonPerformance" = (
        betterproto.message_field(4, group="details")
    )
    time_comparison: "KeyDriverAnalysisResultSegmentTimeComparisonPerformance" = (
        betterproto.message_field(5, group="details")
    )
    general_performance: "KeyDriverAnalysisResultSegmentGeneralPerformance" = (
        betterproto.message_field(6, group="details")
    )
    impact: Optional[float] = betterproto.message_field(
        7, wraps=betterproto.TYPE_DOUBLE
    )
    """
    How much this segment contributes to the overall value of the metric
    calculation.
    """

    confidence: "KeyDriverAnalysisResultSegmentConfidenceLevel" = (
        betterproto.enum_field(8)
    )
    """
    Interpretation of q-value as the confidence of the impact on the metric.
    """

    custom_calculations: List[
        "KeyDriverAnalysisResultSegmentCustomCalculation"
    ] = betterproto.message_field(9)
    """custom calculation statistics."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultSegmentGroupComparisonPerformance(betterproto.Message):
    """
    If analysis type is GROUP_COMPARISON the metric value and size of the
    compared segments.
    """

    group_a_size: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE
    )
    """The size of this segment in the first group."""

    group_b_size: Optional[float] = betterproto.message_field(
        2, wraps=betterproto.TYPE_DOUBLE
    )
    """The size of this segment in the second group."""

    group_a_value: Optional[float] = betterproto.message_field(
        3, wraps=betterproto.TYPE_DOUBLE
    )
    """The value of the metric for this segment in the first group."""

    group_b_value: Optional[float] = betterproto.message_field(
        4, wraps=betterproto.TYPE_DOUBLE
    )
    """The value of the metric for this segment in the second group."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultSegmentTimeComparisonPerformance(betterproto.Message):
    """
    If analysis type is TIME_COMPARISON,  the metric value and size of the
    compared segments.
    """

    previous_period_size: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE
    )
    """The size of this segment in the earlier of the compared periods."""

    recent_period_size: Optional[float] = betterproto.message_field(
        2, wraps=betterproto.TYPE_DOUBLE
    )
    """The size of this segment in the more recent of the compared periods."""

    previous_period_value: Optional[float] = betterproto.message_field(
        3, wraps=betterproto.TYPE_DOUBLE
    )
    """
    The value of the metric for this segment in the earlier of the compared
    periods.
    """

    recent_period_value: Optional[float] = betterproto.message_field(
        4, wraps=betterproto.TYPE_DOUBLE
    )
    """
    The value of the metric for this segment in the more recent of the compared
    periods.
    """


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultSegmentGeneralPerformance(betterproto.Message):
    """
    If analysis type is GENERAL_PERFORMANCE the metric value and size for this
    segment.
    """

    size: Optional[float] = betterproto.message_field(1, wraps=betterproto.TYPE_DOUBLE)
    """
    The size (in percent) of this segment relative to the overall population.
    """

    value: Optional[float] = betterproto.message_field(2, wraps=betterproto.TYPE_DOUBLE)
    """The metric value corresponding to this segment."""


@dataclass(eq=False, repr=False)
class KeyDriverAnalysisResultSegmentCustomCalculation(betterproto.Message):
    calculation_type: "KeyDriverAnalysisResultSegmentCustomCalculationType" = (
        betterproto.enum_field(1)
    )
    """The calculation type of the statistic."""

    name: str = betterproto.string_field(2)
    """The readable name of the statistic."""

    value: Optional[float] = betterproto.message_field(3, wraps=betterproto.TYPE_DOUBLE)
    """The value of the custom calculation statistic."""


@dataclass(eq=False, repr=False)
class TrendAnalysisResult(betterproto.Message):
    """Provides details of a Trend Analysis result."""

    summary_card: "TrendSummaryCardInfo" = betterproto.message_field(1)
    """Summary information for the trend detection analysis."""

    overall_trends: List["TrendAnalysisResultTrend"] = betterproto.message_field(2)
    """Metric level trends."""

    segments: List["TrendAnalysisResultSegment"] = betterproto.message_field(3)
    """Array of the segments in the trend."""


@dataclass(eq=False, repr=False)
class TrendAnalysisResultTrend(betterproto.Message):
    """Provides fields that describes the trend."""

    time_range: "TimeRange" = betterproto.message_field(1)
    """Inclusive start and exclusive end time range."""

    intercept: Optional[float] = betterproto.double_field(
        2, optional=True, group="_intercept"
    )
    """Y-intersept of the trend."""

    trend: Optional[float] = betterproto.double_field(3, optional=True, group="_trend")
    """Steepness of trend."""

    size: Optional[float] = betterproto.double_field(4, optional=True, group="_size")
    """
    The size (in percent) of this trend relative to the overall population.
    """


@dataclass(eq=False, repr=False)
class TrendAnalysisResultSegment(betterproto.Message):
    """Segment of an trend analysis run."""

    id: int = betterproto.int64_field(1)
    """Unique ID corresponding to each segment, unique per analysis run."""

    factors: Dict[str, "Factor"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    The factors that define this segment, represented as a map of Dimension to
    Value.
    """

    trends: List["TrendAnalysisResultTrend"] = betterproto.message_field(4)
    """Trends for the segment."""

    impact: Optional[float] = betterproto.message_field(
        5, wraps=betterproto.TYPE_DOUBLE
    )
    """
    How much this segment contributes to the overall value of the metric
    calculation.
    """


@dataclass(eq=False, repr=False)
class TimeRange(betterproto.Message):
    """TimeRange start and end details with inclusive start and end dates."""

    start_date_inclusive: datetime = betterproto.message_field(1)
    end_date_inclusive: datetime = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Value(betterproto.Message):
    """Represent different possible primitive data types."""

    boolean_value: bool = betterproto.bool_field(1, group="value_type")
    integer_value: int = betterproto.int64_field(2, group="value_type")
    string_value: str = betterproto.string_field(3, group="value_type")
    float_value: float = betterproto.double_field(4, group="value_type")
    timestamp_value: datetime = betterproto.message_field(5, group="value_type")


@dataclass(eq=False, repr=False)
class Factor(betterproto.Message):
    """(Dimension, Value) pairs that define a segment."""

    value: "Value" = betterproto.message_field(1, group="factor_type")
    """Value, it is either string, int, boolean, float or timestamp type."""

    keyword: "FactorKeyword" = betterproto.message_field(2, group="factor_type")
    """Keyword in a text dimension."""

    bin: "FactorBin" = betterproto.message_field(3, group="factor_type")
    """bin of a numerical dimension."""


@dataclass(eq=False, repr=False)
class FactorKeyword(betterproto.Message):
    """Keyword FactorType."""

    keyword: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class FactorBin(betterproto.Message):
    """Bin FactorType."""

    lower_bound: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE
    )
    """The inclusive lower bound of the bin."""

    upper_bound: Optional[float] = betterproto.message_field(
        2, wraps=betterproto.TYPE_DOUBLE
    )
    """The exclusive upper bound of the bin."""

    lower_bound_percentile: Optional[float] = betterproto.message_field(
        3, wraps=betterproto.TYPE_DOUBLE
    )
    """The percentile of `lower_bound` within the factor's dimension."""

    upper_bound_percentile: Optional[float] = betterproto.message_field(
        4, wraps=betterproto.TYPE_DOUBLE
    )
    """The percentile of `upper_bound` within the factor's dimension."""


@dataclass(eq=False, repr=False)
class MetricsListRequest(betterproto.Message):
    """Request payload for get metrics."""

    pass


@dataclass(eq=False, repr=False)
class MetricsListResponse(betterproto.Message):
    """ListMetricsResponse provides list of Metrics."""

    metrics: List["Metric"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UnitsConfig(betterproto.Message):
    type: "UnitsConfigUnitType" = betterproto.enum_field(1)
    label: Optional[str] = betterproto.message_field(2, wraps=betterproto.TYPE_STRING)
    scale: Optional[int] = betterproto.message_field(3, wraps=betterproto.TYPE_INT32)
    precision: Optional[int] = betterproto.message_field(
        4, wraps=betterproto.TYPE_INT32
    )
    currency: Optional[str] = betterproto.message_field(
        5, wraps=betterproto.TYPE_STRING
    )
    kmb: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class Metric(betterproto.Message):
    """Detailed information about a metric."""

    id: int = betterproto.uint64_field(1)
    """Metric id."""

    name: str = betterproto.string_field(2)
    """Metric name."""

    weight_dimension_name: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    The name of the weight dimension. A weight dimension is used to increases
    the importance of a given row in an analysis.
    """

    time_dimension_name: Optional[str] = betterproto.message_field(
        4, wraps=betterproto.TYPE_STRING
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    desired_direction: "MetricDesiredDirection" = betterproto.enum_field(5)
    """
    Specifies whether the metric's goal is to increase or decrease the kpi
    value.
    """

    metric_dimension: "MetricMetricDimension" = betterproto.message_field(6)
    """The dimension which defines the metric's goal."""

    kpi_units_config: Optional["UnitsConfig"] = betterproto.message_field(
        7, optional=True, group="_kpi_units_config"
    )
    """The unit associated with the KDA metric."""

    type: "MetricMetricType" = betterproto.enum_field(8)
    """Type of metric calculation."""

    created_at: datetime = betterproto.message_field(9)
    """Timestamp when the metric was created."""

    dataset_id: int = betterproto.uint64_field(10)
    """Id of the metric data set."""


@dataclass(eq=False, repr=False)
class MetricMetricDimension(betterproto.Message):
    """A dimension in a metric."""

    name: str = betterproto.string_field(1)
    """The name of the dimension."""

    value: "Value" = betterproto.message_field(2)
    """Value, it is either string, int, boolean, float or timestamp type."""


@dataclass(eq=False, repr=False)
class AnalysisDimensionsListRequest(betterproto.Message):
    """Request for a list of dimensions for a given `analysis_id`."""

    analysis_id: int = betterproto.int64_field(1)
    """Unique ID corresponding to the analysis containing the dimensions."""

    is_selected: Optional[str] = betterproto.message_field(
        2, wraps=betterproto.TYPE_STRING
    )
    """
    Will return only active or non active dimensions for a given analysis.
    """


@dataclass(eq=False, repr=False)
class DataSetDimensionsListRequest(betterproto.Message):
    """Request for a list of dimensions for a given `dimension_id`."""

    dataset_id: int = betterproto.int64_field(1)
    """Unique ID corresponding to the dataset containing the dimensions."""


@dataclass(eq=False, repr=False)
class Dimension(betterproto.Message):
    """Represents a dimension (column) within a specfic `Dataset`."""

    name: str = betterproto.string_field(1)
    """
    A string literal corresponding to a column selected in the dataset. It is
    either a column name or a column alias for a computed SQL function over a
    column ( for example, `upper(email) as 'UPPER_EMAIL'` ex: `red` in `SELECT
    roja as red`.
    """

    dataset_id: int = betterproto.uint64_field(2)
    """Refers to the dataset that this dimension is associated with."""

    dimension_type: "SqlDataType" = betterproto.enum_field(3)
    """
    Refers to the column type of this dimension as it exists within the data
    warehouse
    """


@dataclass(eq=False, repr=False)
class AnalysisDimension(betterproto.Message):
    """Represents a dimension (column) within a specfic `analsyis`."""

    name: str = betterproto.string_field(1)
    """
    A string literal corresponding to a column selected in the dataset. It is
    either a column name or a column alias for a computed SQL function over a
    column ( for example, `upper(email) as 'UPPER_EMAIL'` ex: `red` in `SELECT
    roja as red`.
    """

    dimension_type: "SqlDataType" = betterproto.enum_field(2)
    """
    Refers to the column type of this dimension as it exists within the data
    warehouse
    """

    is_selected: bool = betterproto.bool_field(3)
    """Set to true if the dimension is part of the analysis."""


@dataclass(eq=False, repr=False)
class DataSetDimensionsListResponse(betterproto.Message):
    """
    DataSetDimensionsListResponse provides list of Dimensions for a given
    dataset.
    """

    dimensions: List["Dimension"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class AnalysisDimensionsListResponse(betterproto.Message):
    """
    AnalysisDimensionsListResponse provides list of Dimensions for a given
    analysis.
    """

    dimensions: List["AnalysisDimension"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DataSourceListRequest(betterproto.Message):
    """Request payload for get datasources."""

    pass


@dataclass(eq=False, repr=False)
class DataSourceListResponse(betterproto.Message):
    """Response payload for get datasources."""

    data_sources: List["DataSource"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DataSource(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Id of a given data source."""

    name: str = betterproto.string_field(2)
    """Represents the table or query name of a data source."""

    data_source_type: "DataSourceDataSourceType" = betterproto.enum_field(3)
    """Type of the data source."""

    created_at: datetime = betterproto.message_field(4)
    """Time when the data source was created."""

    created_by: str = betterproto.string_field(5)
    """Data source creator's user email info."""

    connection_uri: str = betterproto.string_field(6)
    """JDBC connection URI to the data source location."""


@dataclass(eq=False, repr=False)
class DataSetsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DataSetsResponse(betterproto.Message):
    datasets: List["Dataset"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Dataset(betterproto.Message):
    """Dataset contains instructions on how to query a data source."""

    id: int = betterproto.uint64_field(1)
    """Id of a given dataset."""

    name: str = betterproto.string_field(2)
    """
    Represents the table name for DATASET_TYPE_TABLE or user-defined for
    DATASET_TYPE_QUERY.
    """

    dataset_type: "DatasetDatasetType" = betterproto.enum_field(3)
    """Dataset type represents how data has been obtained."""

    query_string: str = betterproto.string_field(4)
    """A SQL query string for a datasete of type query."""

    last_modified: datetime = betterproto.message_field(5)
    """Last time the data set was modified."""

    created_at: datetime = betterproto.message_field(6)
    """Time when the data set was created."""

    data_source_id: int = betterproto.uint64_field(7)
    """Data Source id of the data set."""


@dataclass(eq=False, repr=False)
class DuplicateAnalysisRequest(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Analysis id to be duplicated"""

    name: str = betterproto.string_field(2)
    """optinal name for the new analysis"""


@dataclass(eq=False, repr=False)
class DuplicateAnalysisResponse(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Newly created Analysis id."""


@dataclass(eq=False, repr=False)
class GetSegmentDataRequest(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Segment id to get data for."""


@dataclass(eq=False, repr=False)
class GetSegmentDataResponse(betterproto.Message):
    query_string: str = betterproto.string_field(1)
    """query string that would result in the segment data."""


@dataclass(eq=False, repr=False)
class CreateDataSetRequest(betterproto.Message):
    dataset: "CreateDataSetRequestDataset" = betterproto.message_field(1)
    data_source_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CreateDataSetRequestDataset(betterproto.Message):
    name: str = betterproto.string_field(1)
    """Name of the dataset."""

    query_string: str = betterproto.string_field(2)
    """A SQL query string for a datasete of type query."""


@dataclass(eq=False, repr=False)
class CreateDataSetResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetProjectsListRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetProjectsListResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetProjectsListResponseListProjectResponse(betterproto.Message):
    projects: List["GetProjectsListResponseProject"] = betterproto.message_field(1)
    """List all projects."""


@dataclass(eq=False, repr=False)
class GetProjectsListResponseProject(betterproto.Message):
    """Provides detailed information about a project."""

    id: int = betterproto.uint64_field(1)
    """Project id."""

    name: str = betterproto.string_field(2)
    """Project name."""

    description: str = betterproto.string_field(3)
    """Project description."""

    created_at: datetime = betterproto.message_field(4)
    """Timestamp when the Project was created."""


@dataclass(eq=False, repr=False)
class GetProjectsAnalysesListRequest(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Project id."""


@dataclass(eq=False, repr=False)
class GetProjectsAnalysesListResponse(betterproto.Message):
    analyses: List["Analysis"] = betterproto.message_field(1)
    """List of analyses associated with a project."""


@dataclass(eq=False, repr=False)
class TimeWindow(betterproto.Message):
    start_date: datetime = betterproto.message_field(1)
    """Start date inclsuvie."""

    end_date: datetime = betterproto.message_field(2)
    """End date exclusive."""


@dataclass(eq=False, repr=False)
class AnalysisDimensionRequest(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CreateAnalysisRequest(betterproto.Message):
    project_id: int = betterproto.uint64_field(1)
    """Project id corresponding to the analysis."""

    name: str = betterproto.string_field(2)
    """The name of the analysis."""

    analysis: "CreateAnalysisRequestAnalysis" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CreateAnalysisRequestAnalysis(betterproto.Message):
    name: str = betterproto.string_field(1)
    """The name of the analysis."""

    type: "AnalysisType" = betterproto.enum_field(2)
    """The Type of the analysis."""

    time_dimension_name: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    metric_id: int = betterproto.uint64_field(4)
    """The metic id which the analysis depends on."""

    filter_expression: "Expression" = betterproto.message_field(5)
    """
    An Expression which would facilitate building a filter expression on the
    analysis.
    """

    time_range: "TimeWindow" = betterproto.message_field(6)
    """A time window the analysis should run on."""

    group_a_name: Optional[str] = betterproto.message_field(
        7, wraps=betterproto.TYPE_STRING
    )
    """The Group a name in a Group compare analysis."""

    group_b_name: Optional[str] = betterproto.message_field(
        8, wraps=betterproto.TYPE_STRING
    )
    """The Group b name in a Group compare analysis."""

    group_a_expression: "Expression" = betterproto.message_field(9)
    """
    A filter expression where group a is denfined in a Group compare analysis.
    """

    group_b_expression: "Expression" = betterproto.message_field(10)
    """
    A filter expression where group b is denfined in a Group compare analysis.
    """

    recent_range: "TimeWindow" = betterproto.message_field(11)
    """Recent time window range used for time compare analysis."""

    previous_range: "TimeWindow" = betterproto.message_field(12)
    """Previous time window range used for time compare analysis."""


@dataclass(eq=False, repr=False)
class CreateAnalysisResponse(betterproto.Message):
    name: str = betterproto.string_field(1)
    """The name of the analysis."""

    id: int = betterproto.int32_field(2)
    """Analysis id."""

    time_dimension_name: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    filter_expression: "Expression" = betterproto.message_field(4)
    """
    An Expression which would facilitate building a filter expression on the
    analysis.
    """

    metric_id: int = betterproto.int32_field(5)
    """The metic id which the analysis depends on."""

    time_range: "TimeWindow" = betterproto.message_field(6)
    """A time window the analysis should run on."""

    group_a_name: Optional[str] = betterproto.message_field(
        7, wraps=betterproto.TYPE_STRING
    )
    """The Group a name in a Group compare analysis."""

    group_b_name: Optional[str] = betterproto.message_field(
        8, wraps=betterproto.TYPE_STRING
    )
    """The Group b name in a Group compare analysis."""

    group_a_expression: "Expression" = betterproto.message_field(9)
    """
    A filter expression where group a is denfined in a Group compare analysis.
    """

    group_b_expression: "Expression" = betterproto.message_field(10)
    """
    A filter expression where group b is denfined in a Group compare analysis.
    """

    project_id: int = betterproto.int32_field(11)
    """Project id corresponding to the analysis."""

    type: "AnalysisType" = betterproto.enum_field(12)
    """The Type of the analysis."""

    created_at: datetime = betterproto.message_field(13)
    """Timestamp when the analysis was created."""

    application_url: str = betterproto.string_field(14)
    """
    Link to the live sisu analysis this represents. ex:
    vip.sisudata.com/projects/{id}/analysis/{id}
    """

    recent_range: "TimeWindow" = betterproto.message_field(15)
    """Recent time window range used for time compare analysis."""

    previous_range: "TimeWindow" = betterproto.message_field(16)
    """Previous time window range used for time compare analysis."""


@dataclass(eq=False, repr=False)
class ModifyAnalysisRequest(betterproto.Message):
    analysis_id: int = betterproto.uint64_field(1)
    analysis: "ModifyAnalysisRequestAnalysis" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ModifyAnalysisRequestAnalysis(betterproto.Message):
    name: Optional[str] = betterproto.string_field(1, optional=True, group="_name")
    """The name of the analysis."""

    time_dimension_name: Optional[Optional[str]] = betterproto.message_field(
        2, wraps=betterproto.TYPE_STRING, optional=True, group="_time_dimension_name"
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    filter_expression: Optional["Expression"] = betterproto.message_field(
        3, optional=True, group="_filter_expression"
    )
    """
    An Expression which would facilitate building a filter expression on the
    analysis.
    """

    time_range: Optional["TimeWindow"] = betterproto.message_field(
        4, optional=True, group="_time_range"
    )
    """A time window the analysis should run on."""

    group_a_name: Optional[Optional[str]] = betterproto.message_field(
        5, wraps=betterproto.TYPE_STRING, optional=True, group="_group_a_name"
    )
    """The Group a name in a Group compare analysis."""

    group_b_name: Optional[Optional[str]] = betterproto.message_field(
        6, wraps=betterproto.TYPE_STRING, optional=True, group="_group_b_name"
    )
    """The Group b name in a Group compare analysis."""

    group_a_expression: Optional["Expression"] = betterproto.message_field(
        7, optional=True, group="_group_a_expression"
    )
    """
    A filter expression where group a is denfined in a Group compare analysis.
    """

    group_b_expression: Optional["Expression"] = betterproto.message_field(
        8, optional=True, group="_group_b_expression"
    )
    """
    A filter expression where group b is denfined in a Group compare analysis.
    """

    recent_range: Optional["TimeWindow"] = betterproto.message_field(
        9, optional=True, group="_recent_range"
    )
    """Recent time window range used for time compare analysis."""

    previous_range: Optional["TimeWindow"] = betterproto.message_field(
        10, optional=True, group="_previous_range"
    )
    """Previous time window range used for time compare analysis."""


@dataclass(eq=False, repr=False)
class ModifyAnalysisResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class UpdateAnalysisRequest(betterproto.Message):
    analysis_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)
    """The name of the analysis."""

    analysis: "UpdateAnalysisRequestAnalysis" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class UpdateAnalysisRequestAnalysis(betterproto.Message):
    name: str = betterproto.string_field(1)
    """The name of the analysis."""

    type: "AnalysisType" = betterproto.enum_field(2)
    """The Type of the analysis."""

    time_dimension_name: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    filter_expression: "Expression" = betterproto.message_field(4)
    """
    An Expression which would facilitate building a filter expression on the
    analysis.
    """

    time_range: "TimeWindow" = betterproto.message_field(5)
    """A time window the analysis should run on."""

    group_a_name: Optional[str] = betterproto.message_field(
        6, wraps=betterproto.TYPE_STRING
    )
    """The Group a name in a Group compare analysis."""

    group_b_name: Optional[str] = betterproto.message_field(
        7, wraps=betterproto.TYPE_STRING
    )
    """The Group b name in a Group compare analysis."""

    group_a_expression: "Expression" = betterproto.message_field(8)
    """
    A filter expression where group a is denfined in a Group compare analysis.
    """

    group_b_expression: "Expression" = betterproto.message_field(9)
    """
    A filter expression where group b is denfined in a Group compare analysis.
    """

    recent_range: "TimeWindow" = betterproto.message_field(10)
    """Recent time window range used for time compare analysis."""

    previous_range: "TimeWindow" = betterproto.message_field(11)
    """Previous time window range used for time compare analysis."""


@dataclass(eq=False, repr=False)
class UpdateAnalysisResponse(betterproto.Message):
    name: str = betterproto.string_field(1)
    """The name of the analysis."""

    id: int = betterproto.uint64_field(2)
    """Analysis id."""

    time_dimension_name: Optional[str] = betterproto.message_field(
        3, wraps=betterproto.TYPE_STRING
    )
    """
    The name of metric's time dimension which represnts the date range of the
    metric.
    """

    filter_expression: "Expression" = betterproto.message_field(4)
    """
    An Expression which would facilitate building a filter expression on the
    analysis.
    """

    metric_id: int = betterproto.uint64_field(5)
    """The metic id which the analysis depends on."""

    time_range: "TimeWindow" = betterproto.message_field(6)
    """A time window the analysis should run on."""

    group_a_name: Optional[str] = betterproto.message_field(
        7, wraps=betterproto.TYPE_STRING
    )
    """The Group a name in a Group compare analysis."""

    group_b_name: Optional[str] = betterproto.message_field(
        8, wraps=betterproto.TYPE_STRING
    )
    """The Group b name in a Group compare analysis."""

    group_a_expression: "Expression" = betterproto.message_field(9)
    """
    A filter expression where group a is denfined in a Group compare analysis.
    """

    group_b_expression: "Expression" = betterproto.message_field(10)
    """
    A filter expression where group b is denfined in a Group compare analysis.
    """

    project_id: int = betterproto.uint64_field(11)
    """Project id corresponding to the analysis."""

    type: "AnalysisType" = betterproto.enum_field(12)
    """The Type of the analysis."""

    created_at: datetime = betterproto.message_field(13)
    """Timestamp when the analysis was created."""

    application_url: str = betterproto.string_field(14)
    """
    Link to the live sisu analysis this represents. ex:
    vip.sisudata.com/projects/{id}/analysis/{id}
    """

    recent_range: "TimeWindow" = betterproto.message_field(15)
    """Recent time window range used for time compare analysis."""

    previous_range: "TimeWindow" = betterproto.message_field(16)
    """Previous time window range used for time compare analysis."""


@dataclass(eq=False, repr=False)
class WaterfallStep(betterproto.Message):
    cumulative_impact_before_step: float = betterproto.double_field(1)
    """The cummulative value of impact prior to the waterfall step."""

    cumulative_impact_after_step: float = betterproto.double_field(2)
    """The cummulative value of impact after to the waterfall step."""

    overlapping_impact: float = betterproto.double_field(3)
    """The value of overlapping impact."""

    change_in_size: "WaterfallStepChangeInSize" = betterproto.message_field(4)
    change_in_type: "WaterfallStepChangeInType" = betterproto.message_field(5)
    factors: Dict[str, "Factor"] = betterproto.map_field(
        6, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Map of Factors for the waterfall step."""


@dataclass(eq=False, repr=False)
class WaterfallStepChangeInType(betterproto.Message):
    """The subgroup change in value."""

    subgroup_a: float = betterproto.double_field(1)
    """previous_period or group_a."""

    subgroup_b: float = betterproto.double_field(2)
    """recent_period or group_b."""


@dataclass(eq=False, repr=False)
class WaterfallStepChangeInSize(betterproto.Message):
    """The subgroup change in value."""

    subgroup_a: float = betterproto.double_field(1)
    """previous_period or group_a."""

    subgroup_b: float = betterproto.double_field(2)
    """recent_period or group_b."""


@dataclass(eq=False, repr=False)
class WaterfallAnalysisResponse(betterproto.Message):
    waterfall: List["WaterfallStep"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WaterfallAnalysisRequest(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    """Analysis id"""


class AnalysesServiceStub(betterproto.ServiceStub):
    async def analyses_list(
        self,
        analyses_list_request: "AnalysesListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AnalysesListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/AnalysesList",
            analyses_list_request,
            AnalysesListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def run_analysis(
        self,
        run_analysis_request: "RunAnalysisRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "RunAnalysisResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/RunAnalysis",
            run_analysis_request,
            RunAnalysisResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def modify_analysis(
        self,
        modify_analysis_request: "ModifyAnalysisRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ModifyAnalysisResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/ModifyAnalysis",
            modify_analysis_request,
            ModifyAnalysisResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def update_analysis(
        self,
        update_analysis_request: "UpdateAnalysisRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "UpdateAnalysisResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/UpdateAnalysis",
            update_analysis_request,
            UpdateAnalysisResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def waterfall_analysis(
        self,
        waterfall_analysis_request: "WaterfallAnalysisRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "WaterfallAnalysisResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/WaterfallAnalysis",
            waterfall_analysis_request,
            WaterfallAnalysisResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def analysis_run_results(
        self,
        analysis_run_results_request: "AnalysisRunResultsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AnalysisRunResultsResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/AnalysisRunResults",
            analysis_run_results_request,
            AnalysisRunResultsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_analysis_filters(
        self,
        get_analysis_filters_request: "GetAnalysisFiltersRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetAnalysisFiltersResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/GetAnalysisFilters",
            get_analysis_filters_request,
            GetAnalysisFiltersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def set_analysis_filters(
        self,
        set_analysis_filters_request: "SetAnalysisFiltersRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SetAnalysisFiltersResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/SetAnalysisFilters",
            set_analysis_filters_request,
            SetAnalysisFiltersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def analysis_dimensions_list(
        self,
        analysis_dimensions_list_request: "AnalysisDimensionsListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AnalysisDimensionsListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/AnalysisDimensionsList",
            analysis_dimensions_list_request,
            AnalysisDimensionsListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def duplicate_analysis(
        self,
        duplicate_analysis_request: "DuplicateAnalysisRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DuplicateAnalysisResponse":
        return await self._unary_unary(
            "/sisu.v1.api.AnalysesService/DuplicateAnalysis",
            duplicate_analysis_request,
            DuplicateAnalysisResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MetricServiceStub(betterproto.ServiceStub):
    async def metrics_list(
        self,
        metrics_list_request: "MetricsListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MetricsListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.MetricService/MetricsList",
            metrics_list_request,
            MetricsListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class DatasetServiceStub(betterproto.ServiceStub):
    async def data_sets(
        self,
        data_sets_request: "DataSetsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DataSetsResponse":
        return await self._unary_unary(
            "/sisu.v1.api.DatasetService/DataSets",
            data_sets_request,
            DataSetsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def data_set_dimensions_list(
        self,
        data_set_dimensions_list_request: "DataSetDimensionsListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DataSetDimensionsListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.DatasetService/DataSetDimensionsList",
            data_set_dimensions_list_request,
            DataSetDimensionsListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class DataSourcesServiceStub(betterproto.ServiceStub):
    async def data_source_list(
        self,
        data_source_list_request: "DataSourceListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DataSourceListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.DataSourcesService/DataSourceList",
            data_source_list_request,
            DataSourceListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def create_data_set(
        self,
        create_data_set_request: "CreateDataSetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "CreateDataSetResponse":
        return await self._unary_unary(
            "/sisu.v1.api.DataSourcesService/CreateDataSet",
            create_data_set_request,
            CreateDataSetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class SegmentsServiceStub(betterproto.ServiceStub):
    async def get_segment_data(
        self,
        get_segment_data_request: "GetSegmentDataRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetSegmentDataResponse":
        return await self._unary_unary(
            "/sisu.v1.api.SegmentsService/GetSegmentData",
            get_segment_data_request,
            GetSegmentDataResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ProjectsServiceStub(betterproto.ServiceStub):
    async def get_projects_list(
        self,
        get_projects_list_request: "GetProjectsListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetProjectsListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.ProjectsService/GetProjectsList",
            get_projects_list_request,
            GetProjectsListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_projects_analyses_list(
        self,
        get_projects_analyses_list_request: "GetProjectsAnalysesListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetProjectsAnalysesListResponse":
        return await self._unary_unary(
            "/sisu.v1.api.ProjectsService/GetProjectsAnalysesList",
            get_projects_analyses_list_request,
            GetProjectsAnalysesListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class AnalysesServiceBase(ServiceBase):
    async def analyses_list(
        self, analyses_list_request: "AnalysesListRequest"
    ) -> "AnalysesListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def run_analysis(
        self, run_analysis_request: "RunAnalysisRequest"
    ) -> "RunAnalysisResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def modify_analysis(
        self, modify_analysis_request: "ModifyAnalysisRequest"
    ) -> "ModifyAnalysisResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_analysis(
        self, update_analysis_request: "UpdateAnalysisRequest"
    ) -> "UpdateAnalysisResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def waterfall_analysis(
        self, waterfall_analysis_request: "WaterfallAnalysisRequest"
    ) -> "WaterfallAnalysisResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def analysis_run_results(
        self, analysis_run_results_request: "AnalysisRunResultsRequest"
    ) -> "AnalysisRunResultsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_analysis_filters(
        self, get_analysis_filters_request: "GetAnalysisFiltersRequest"
    ) -> "GetAnalysisFiltersResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_analysis_filters(
        self, set_analysis_filters_request: "SetAnalysisFiltersRequest"
    ) -> "SetAnalysisFiltersResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def analysis_dimensions_list(
        self, analysis_dimensions_list_request: "AnalysisDimensionsListRequest"
    ) -> "AnalysisDimensionsListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def duplicate_analysis(
        self, duplicate_analysis_request: "DuplicateAnalysisRequest"
    ) -> "DuplicateAnalysisResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_analyses_list(
        self, stream: "grpclib.server.Stream[AnalysesListRequest, AnalysesListResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.analyses_list(request)
        await stream.send_message(response)

    async def __rpc_run_analysis(
        self, stream: "grpclib.server.Stream[RunAnalysisRequest, RunAnalysisResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.run_analysis(request)
        await stream.send_message(response)

    async def __rpc_modify_analysis(
        self,
        stream: "grpclib.server.Stream[ModifyAnalysisRequest, ModifyAnalysisResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.modify_analysis(request)
        await stream.send_message(response)

    async def __rpc_update_analysis(
        self,
        stream: "grpclib.server.Stream[UpdateAnalysisRequest, UpdateAnalysisResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.update_analysis(request)
        await stream.send_message(response)

    async def __rpc_waterfall_analysis(
        self,
        stream: "grpclib.server.Stream[WaterfallAnalysisRequest, WaterfallAnalysisResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.waterfall_analysis(request)
        await stream.send_message(response)

    async def __rpc_analysis_run_results(
        self,
        stream: "grpclib.server.Stream[AnalysisRunResultsRequest, AnalysisRunResultsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.analysis_run_results(request)
        await stream.send_message(response)

    async def __rpc_get_analysis_filters(
        self,
        stream: "grpclib.server.Stream[GetAnalysisFiltersRequest, GetAnalysisFiltersResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_analysis_filters(request)
        await stream.send_message(response)

    async def __rpc_set_analysis_filters(
        self,
        stream: "grpclib.server.Stream[SetAnalysisFiltersRequest, SetAnalysisFiltersResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.set_analysis_filters(request)
        await stream.send_message(response)

    async def __rpc_analysis_dimensions_list(
        self,
        stream: "grpclib.server.Stream[AnalysisDimensionsListRequest, AnalysisDimensionsListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.analysis_dimensions_list(request)
        await stream.send_message(response)

    async def __rpc_duplicate_analysis(
        self,
        stream: "grpclib.server.Stream[DuplicateAnalysisRequest, DuplicateAnalysisResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.duplicate_analysis(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.AnalysesService/AnalysesList": grpclib.const.Handler(
                self.__rpc_analyses_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                AnalysesListRequest,
                AnalysesListResponse,
            ),
            "/sisu.v1.api.AnalysesService/RunAnalysis": grpclib.const.Handler(
                self.__rpc_run_analysis,
                grpclib.const.Cardinality.UNARY_UNARY,
                RunAnalysisRequest,
                RunAnalysisResponse,
            ),
            "/sisu.v1.api.AnalysesService/ModifyAnalysis": grpclib.const.Handler(
                self.__rpc_modify_analysis,
                grpclib.const.Cardinality.UNARY_UNARY,
                ModifyAnalysisRequest,
                ModifyAnalysisResponse,
            ),
            "/sisu.v1.api.AnalysesService/UpdateAnalysis": grpclib.const.Handler(
                self.__rpc_update_analysis,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateAnalysisRequest,
                UpdateAnalysisResponse,
            ),
            "/sisu.v1.api.AnalysesService/WaterfallAnalysis": grpclib.const.Handler(
                self.__rpc_waterfall_analysis,
                grpclib.const.Cardinality.UNARY_UNARY,
                WaterfallAnalysisRequest,
                WaterfallAnalysisResponse,
            ),
            "/sisu.v1.api.AnalysesService/AnalysisRunResults": grpclib.const.Handler(
                self.__rpc_analysis_run_results,
                grpclib.const.Cardinality.UNARY_UNARY,
                AnalysisRunResultsRequest,
                AnalysisRunResultsResponse,
            ),
            "/sisu.v1.api.AnalysesService/GetAnalysisFilters": grpclib.const.Handler(
                self.__rpc_get_analysis_filters,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetAnalysisFiltersRequest,
                GetAnalysisFiltersResponse,
            ),
            "/sisu.v1.api.AnalysesService/SetAnalysisFilters": grpclib.const.Handler(
                self.__rpc_set_analysis_filters,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetAnalysisFiltersRequest,
                SetAnalysisFiltersResponse,
            ),
            "/sisu.v1.api.AnalysesService/AnalysisDimensionsList": grpclib.const.Handler(
                self.__rpc_analysis_dimensions_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                AnalysisDimensionsListRequest,
                AnalysisDimensionsListResponse,
            ),
            "/sisu.v1.api.AnalysesService/DuplicateAnalysis": grpclib.const.Handler(
                self.__rpc_duplicate_analysis,
                grpclib.const.Cardinality.UNARY_UNARY,
                DuplicateAnalysisRequest,
                DuplicateAnalysisResponse,
            ),
        }


class MetricServiceBase(ServiceBase):
    async def metrics_list(
        self, metrics_list_request: "MetricsListRequest"
    ) -> "MetricsListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_metrics_list(
        self, stream: "grpclib.server.Stream[MetricsListRequest, MetricsListResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.metrics_list(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.MetricService/MetricsList": grpclib.const.Handler(
                self.__rpc_metrics_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                MetricsListRequest,
                MetricsListResponse,
            ),
        }


class DatasetServiceBase(ServiceBase):
    async def data_sets(
        self, data_sets_request: "DataSetsRequest"
    ) -> "DataSetsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def data_set_dimensions_list(
        self, data_set_dimensions_list_request: "DataSetDimensionsListRequest"
    ) -> "DataSetDimensionsListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_data_sets(
        self, stream: "grpclib.server.Stream[DataSetsRequest, DataSetsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.data_sets(request)
        await stream.send_message(response)

    async def __rpc_data_set_dimensions_list(
        self,
        stream: "grpclib.server.Stream[DataSetDimensionsListRequest, DataSetDimensionsListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.data_set_dimensions_list(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.DatasetService/DataSets": grpclib.const.Handler(
                self.__rpc_data_sets,
                grpclib.const.Cardinality.UNARY_UNARY,
                DataSetsRequest,
                DataSetsResponse,
            ),
            "/sisu.v1.api.DatasetService/DataSetDimensionsList": grpclib.const.Handler(
                self.__rpc_data_set_dimensions_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                DataSetDimensionsListRequest,
                DataSetDimensionsListResponse,
            ),
        }


class DataSourcesServiceBase(ServiceBase):
    async def data_source_list(
        self, data_source_list_request: "DataSourceListRequest"
    ) -> "DataSourceListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_data_set(
        self, create_data_set_request: "CreateDataSetRequest"
    ) -> "CreateDataSetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_data_source_list(
        self,
        stream: "grpclib.server.Stream[DataSourceListRequest, DataSourceListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.data_source_list(request)
        await stream.send_message(response)

    async def __rpc_create_data_set(
        self,
        stream: "grpclib.server.Stream[CreateDataSetRequest, CreateDataSetResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_data_set(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.DataSourcesService/DataSourceList": grpclib.const.Handler(
                self.__rpc_data_source_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                DataSourceListRequest,
                DataSourceListResponse,
            ),
            "/sisu.v1.api.DataSourcesService/CreateDataSet": grpclib.const.Handler(
                self.__rpc_create_data_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateDataSetRequest,
                CreateDataSetResponse,
            ),
        }


class SegmentsServiceBase(ServiceBase):
    async def get_segment_data(
        self, get_segment_data_request: "GetSegmentDataRequest"
    ) -> "GetSegmentDataResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_segment_data(
        self,
        stream: "grpclib.server.Stream[GetSegmentDataRequest, GetSegmentDataResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_segment_data(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.SegmentsService/GetSegmentData": grpclib.const.Handler(
                self.__rpc_get_segment_data,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetSegmentDataRequest,
                GetSegmentDataResponse,
            ),
        }


class ProjectsServiceBase(ServiceBase):
    async def get_projects_list(
        self, get_projects_list_request: "GetProjectsListRequest"
    ) -> "GetProjectsListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_projects_analyses_list(
        self, get_projects_analyses_list_request: "GetProjectsAnalysesListRequest"
    ) -> "GetProjectsAnalysesListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_projects_list(
        self,
        stream: "grpclib.server.Stream[GetProjectsListRequest, GetProjectsListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_projects_list(request)
        await stream.send_message(response)

    async def __rpc_get_projects_analyses_list(
        self,
        stream: "grpclib.server.Stream[GetProjectsAnalysesListRequest, GetProjectsAnalysesListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_projects_analyses_list(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/sisu.v1.api.ProjectsService/GetProjectsList": grpclib.const.Handler(
                self.__rpc_get_projects_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetProjectsListRequest,
                GetProjectsListResponse,
            ),
            "/sisu.v1.api.ProjectsService/GetProjectsAnalysesList": grpclib.const.Handler(
                self.__rpc_get_projects_analyses_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetProjectsAnalysesListRequest,
                GetProjectsAnalysesListResponse,
            ),
        }



from unittest import mock

from pysisu.pysisu_class import PySisu
from pysisu.proto.sisu.v1.api import (
    AnalysisRunResultsResponse,
    AnalysesListResponse,
    DataSetsResponse,
    DataSourceListResponse,
    GetAnalysisFiltersResponse,
    GetProjectsAnalysesListResponse,
    GetProjectsListResponseListProjectResponse,
    MetricsListResponse,
    PaginationHints,
)
from pytest import fixture

from pysisu.tests.helpers_for_tests import (
    INPUT_SNAPSHOT_DIR,
    OUTPUT_SNAPSHOT_DIR,
    load_snapshot,
    pyapp_api_snapshots_to_pysisu,
)


def pytest_collection_finish(session):
    # ensure we have latest API response snapshots from the pyapp
    # do after collection so this step happens even if run from root pysisu directory
    pyapp_api_snapshots_to_pysisu()


@fixture
def pysisu_input_snapshot(snapshot):
    snapshot.snapshot_dir = INPUT_SNAPSHOT_DIR
    return snapshot


@fixture
def pysisu_output_snapshot(snapshot):
    snapshot.snapshot_dir = OUTPUT_SNAPSHOT_DIR
    return snapshot


@fixture
@mock.patch.object(PySisu, "_check_compatibility", lambda x: None)
def pysisu_client():
    pysisu = PySisu("API_KEY", "URL")
    return pysisu


@fixture
def result_time_compare() -> AnalysisRunResultsResponse:
    return AnalysisRunResultsResponse().from_dict(
        load_snapshot("time compare")
    )


@fixture
def result_group_compare() -> AnalysisRunResultsResponse:
    return AnalysisRunResultsResponse().from_dict(
        load_snapshot("group compare")
    )


@fixture
def result_general_performance() -> AnalysisRunResultsResponse:
    return AnalysisRunResultsResponse().from_dict(
        load_snapshot("general performance")
    )


@fixture
def result_trend() -> AnalysisRunResultsResponse:
    return AnalysisRunResultsResponse().from_dict(load_snapshot("trend"))


@fixture
def result_has_more(result_time_compare) -> AnalysisRunResultsResponse:
    # make a copy of result_time_compare
    result = AnalysisRunResultsResponse().from_dict(
        result_time_compare.to_dict()
    )
    result.pagination_hints = PaginationHints(
        has_more=True, next_starting_cursor=1
    )
    return result


@fixture
def analyses_list() -> AnalysesListResponse:
    return AnalysesListResponse().from_dict(load_snapshot("list analyses"))


@fixture
def metrics_list() -> MetricsListResponse:
    return MetricsListResponse().from_dict(load_snapshot("list metrics"))


@fixture
def data_sources_list() -> DataSourceListResponse:
    return DataSourceListResponse().from_dict(
        load_snapshot("list data sources")
    )


@fixture
def datasets_list() -> DataSetsResponse:
    return DataSetsResponse().from_dict(load_snapshot("list datasets"))


@fixture
def analysis_filter() -> GetAnalysisFiltersResponse:
    return GetAnalysisFiltersResponse().from_dict(
        load_snapshot("get analysis filters")
    )


@fixture
def projects_list() -> GetProjectsListResponseListProjectResponse:
    return GetProjectsListResponseListProjectResponse().from_dict(
        load_snapshot("list projects")
    )


@fixture
def project_analyses_list() -> GetProjectsAnalysesListResponse:
    return GetProjectsAnalysesListResponse().from_dict(
        load_snapshot("list project analyses")
    )
